/*
* ============================================================================
* History
* =======
*
* (C) Copyright 2014, Future Technology Devices International Ltd.
* ============================================================================
*
* This source code ("the Software") is provided by Future Technology Devices
* International Limited ("FTDI") subject to the licence terms set out
* http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
* You must read the Licence Terms before downloading or using the Software.
* By installing or using the Software you agree to the Licence Terms. If you
* do not agree to the Licence Terms then do not download or use the Software.
*
* Without prejudice to the Licence Terms, here is a summary of some of the key
* terms of the Licence Terms (and in the event of any conflict between this
* summary and the Licence Terms then the text of the Licence Terms will
* prevail).
*
* The Software is provided "as is".
* There are no warranties (or similar) in relation to the quality of the
* Software. You use it at your own risk.
* The Software should not be used in, or for, any medical device, system or
* appliance. There are exclusions of FTDI liability for certain types of loss
* such as: special loss or damage; incidental loss or damage; indirect or
* consequential loss or damage; loss of income; loss of business; loss of
* profits; loss of revenue; loss of contracts; business interruption; loss of
* the use of money or anticipated savings; loss of information; loss of
* opportunity; loss of goodwill or reputation; and/or loss of, damage to or
* corruption of data.
* There is a monetary cap on FTDI's liability.
* The Software may have subsequently been amended by another user and then
* distributed by that other user ("Adapted Software").  If so that user may
* have additional licence terms that apply to those amendments. However, FTDI
* has no liability in relation to those amendments.
* ============================================================================
*/
#ifndef _FT_HAL_SPI_H_
#define _FT_HAL_SPI_H_

/* APIs for SPI HAL initialization */
ft_void hal_spi_init();
ft_void hal_spi_exit();

/* APIs for read/write start */
ft_void hal_spi_rdstart(ft_uint32_t addr);
ft_void hal_spi_wrstart(ft_uint32_t addr);
ft_void  hal_spi_wrcmdstart(ft_int16_t count) ;
/* API for read/write end */
ft_void hal_spi_end();

/* APIs for read functionality */
ft_uint8_t hal_spi_rd8(ft_uint32_t addr);
ft_uint16_t hal_spi_rd16(ft_uint32_t addr);
ft_uint32_t hal_spi_rd32(ft_uint32_t addr);
/* returns number of bytes read */
ft_uint32_t hal_spi_rd8s(ft_uint32_t addr,ft_uint8_t *dst,ft_uint32_t count);

/* APIs for write functionality */
ft_void hal_spi_wr8(ft_uint32_t addr, ft_uint8_t v);
ft_void hal_spi_wr16(ft_uint32_t addr, ft_uint16_t v);
ft_void hal_spi_wr32(ft_uint32_t addr, ft_uint32_t v);

ft_void hal_spi_wr8s(ft_uint32_t addr,ft_uint8_t *src, ft_uint32_t count);

/* APIs for transfer functionality - no CS enable and disable done in these APIs */
ft_void hal_spi_transfer_string(ft_char8_t *s);
ft_void hal_spi_transfer8(ft_uint8_t v);
ft_void hal_spi_transfer16(ft_uint16_t v);
ft_void hal_spi_transfer32(ft_uint32_t v);

/* Coprocessor related APIs */
ft_int16_t hal_get_cmdfifo_freespace();
ft_void hal_spi_updatecmdfifo(ft_uint16_t count);
ft_void hal_spi_wrcmd32(ft_uint32_t c);
ft_void hal_spi_waitcmdfifo_empty();

ft_void hal_spi_cmdidle();
ft_void hal_spi_wrcmd32s(FT_PROGMEM prog_uchar *src, ft_uint32_t count);
ft_void hal_spi_download_jpeg(FT_PROGMEM prog_uchar *src, ft_uint16_t count);

ft_void hal_gpu_dl(ft_uint32_t cmd);

/* Co processor dl specific apis */
ft_void hal_dl_start();
ft_void hal_dl(ft_uint32_t c);
ft_void hal_dl_end();

ft_uint8_t hal_spi_receive8();
ft_void hal_spi_insertdummy();
ft_uint8_t SPI_Transfer_Byte(ft_uint8_t wrbyte);
ft_void hal_spi_clkconfig();
ft_void hal_spi_corerst();
ft_void hal_spi_wrcmd32s_nowaitidle(FT_PROGMEM prog_uchar *src, ft_uint32_t count);

ft_void hal_spi_resetwp();
ft_void SPI_CS_SD_Assert();
ft_void SPI_CS_SD_Dessert();

/* Below to be part of gpu utils */
ft_void fadein();
ft_void fadeout();

/* String related Apis - ideally to be part of PAL */
ft_void hal_spi_transfer_string_const(const ft_char8_t *s);
ft_void hal_spi_transfer_string(ft_char8_t *s);
ft_char8_t hal_toupper(ft_char8_t inputchar);
ft_char8_t hal_memcmp_sram(ft_char8_t *p1,ft_char8_t *p2,ft_int32_t num);
ft_char8_t hal_memcmp(ft_char8_t *p1,const ft_char8_t *p2,ft_int32_t num);
ft_char8_t* hal_strcpy(ft_char8_t *dst, const ft_char8_t *src);
ft_int32_t hal_strlen(const ft_char8_t *s);

#endif /* _FT_HAL_SPI_H_ */


/*  Nothing beyond this */


