/*
* ============================================================================
* History
* =======
*
* (C) Copyright 2014, Future Technology Devices International Ltd.
* ============================================================================
*
* This source code ("the Software") is provided by Future Technology Devices
* International Limited ("FTDI") subject to the licence terms set out
* http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
* You must read the Licence Terms before downloading or using the Software.
* By installing or using the Software you agree to the Licence Terms. If you
* do not agree to the Licence Terms then do not download or use the Software.
*
* Without prejudice to the Licence Terms, here is a summary of some of the key
* terms of the Licence Terms (and in the event of any conflict between this
* summary and the Licence Terms then the text of the Licence Terms will
* prevail).
*
* The Software is provided "as is".
* There are no warranties (or similar) in relation to the quality of the
* Software. You use it at your own risk.
* The Software should not be used in, or for, any medical device, system or
* appliance. There are exclusions of FTDI liability for certain types of loss
* such as: special loss or damage; incidental loss or damage; indirect or
* consequential loss or damage; loss of income; loss of business; loss of
* profits; loss of revenue; loss of contracts; business interruption; loss of
* the use of money or anticipated savings; loss of information; loss of
* opportunity; loss of goodwill or reputation; and/or loss of, damage to or
* corruption of data.
* There is a monetary cap on FTDI's liability.
* The Software may have subsequently been amended by another user and then
* distributed by that other user ("Adapted Software").  If so that user may
* have additional licence terms that apply to those amendments. However, FTDI
* has no liability in relation to those amendments.
* ============================================================================
*/

#include "FT_DataTypes.h"
#include "FT_Hal_Spi.h"
#include "FT_Gpu.h"
/* APIs for j1 commands */


ft_void hal_spi_cmd_text(ft_int16_t x, ft_int16_t y, ft_int16_t font, ft_uint16_t options, const ft_char8_t* s)
{
  ft_int16_t stringlen = 0;
  stringlen = hal_strlen(s);
  hal_spi_wrcmdstart(4 + 2 + 2 + 2 + 2 + stringlen + 1);
  hal_spi_transfer32(CMD_TEXT);
  hal_spi_transfer16(x);
  hal_spi_transfer16(y);
  hal_spi_transfer16(font);
  hal_spi_transfer16(options);
  hal_spi_transfer_string_const(s);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 2 + 2 + 2 + 2 + stringlen + 1);
}
ft_void hal_spi_cmd_number(ft_int16_t x, ft_int16_t y, ft_int16_t font, ft_uint16_t options, ft_int32_t n)
{
  hal_spi_wrcmdstart(4 + 2 + 2 + 2 + 2 + 4);
  hal_spi_transfer32(CMD_NUMBER);
  hal_spi_transfer16(x);
  hal_spi_transfer16(y);
  hal_spi_transfer16(font);
  hal_spi_transfer16(options);
  hal_spi_transfer32(n);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 2 + 2 + 2 + 2 + 4);
}
ft_void hal_spi_cmd_loadidentity()
{
  hal_spi_wrcmdstart(4);
  hal_spi_transfer32(CMD_LOADIDENTITY);
  hal_spi_end();
  hal_spi_updatecmdfifo(4);
}
ft_void hal_spi_cmd_toggle(ft_int16_t x, ft_int16_t y, ft_int16_t w, ft_int16_t font, ft_uint16_t options, ft_uint16_t state, const ft_char8_t* s)
{
  hal_spi_wrcmdstart(4 + 2 + 2 + 2 + 2 + 2 + 2 + hal_strlen(s) + 1);
  hal_spi_transfer32(CMD_TOGGLE);
  hal_spi_transfer16(x);
  hal_spi_transfer16(y);
  hal_spi_transfer16(w);
  hal_spi_transfer16(font);
  hal_spi_transfer16(options);
  hal_spi_transfer16(state);
  hal_spi_transfer_string_const(s);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 2 + 2 + 2 + 2 + 2 + 2 + hal_strlen(s) + 1);
}
ft_void hal_spi_cmd_gauge(ft_int16_t x, ft_int16_t y, ft_int16_t r, ft_uint16_t options, ft_uint16_t major, ft_uint16_t minor, ft_uint16_t val, ft_uint16_t range)
{
  hal_spi_wrcmdstart(4 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2);
  hal_spi_transfer32(CMD_GAUGE);
  hal_spi_transfer16(x);
  hal_spi_transfer16(y);
  hal_spi_transfer16(r);
  hal_spi_transfer16(options);
  hal_spi_transfer16(major);
  hal_spi_transfer16(minor);
  hal_spi_transfer16(val);
  hal_spi_transfer16(range);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2);
}
ft_void hal_spi_cmd_regread(ft_uint32_t ptr, ft_uint32_t result)
{
  hal_spi_wrcmdstart(4 + 4 + 4);
  hal_spi_transfer32(CMD_REGREAD);
  hal_spi_transfer32(ptr);
  hal_spi_transfer32(result);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4 + 4);
}
ft_void hal_spi_cmd_getprops(ft_uint32_t ptr, ft_uint32_t w, ft_uint32_t h)
{
  hal_spi_wrcmdstart(4 + 4 + 4 + 4);
  hal_spi_transfer32(CMD_GETPROPS);
  hal_spi_transfer32(ptr);
  hal_spi_transfer32(w);
  hal_spi_transfer32(h);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4 + 4 + 4);
}
ft_void hal_spi_cmd_memcpy(ft_uint32_t dest, ft_uint32_t src, ft_uint32_t num)
{
  hal_spi_wrcmdstart(4 + 4 + 4 + 4);
  hal_spi_transfer32(CMD_MEMCPY);
  hal_spi_transfer32(dest);
  hal_spi_transfer32(src);
  hal_spi_transfer32(num);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4 + 4 + 4);
}
ft_void hal_spi_cmd_spinner(ft_int16_t x, ft_int16_t y, ft_uint16_t style, ft_uint16_t scale)
{
  hal_spi_wrcmdstart(4 + 2 + 2 + 2 + 2);
  hal_spi_transfer32(CMD_SPINNER);
  hal_spi_transfer16(x);
  hal_spi_transfer16(y);
  hal_spi_transfer16(style);
  hal_spi_transfer16(scale);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 2 + 2 + 2 + 2);
}
ft_void hal_spi_cmd_bgcolor(ft_uint32_t c)
{
  hal_spi_wrcmdstart(4 + 4);
  hal_spi_transfer32(CMD_BGCOLOR);
  hal_spi_transfer32(c);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4);
}
ft_void hal_spi_cmd_swap()
{
  hal_spi_wrcmdstart(4);
  hal_spi_transfer32(CMD_SWAP);
  hal_spi_end();
  hal_spi_updatecmdfifo(4);
}
ft_void hal_spi_cmd_inflate(ft_uint32_t ptr)
{
  hal_spi_wrcmdstart(4 + 4);
  hal_spi_transfer32(CMD_INFLATE);
  hal_spi_transfer32(ptr);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4);
}
ft_void hal_spi_cmd_translate(ft_int32_t tx, ft_int32_t ty)
{
  hal_spi_wrcmdstart(4 + 4 + 4);
  hal_spi_transfer32(CMD_TRANSLATE);
  hal_spi_transfer32(tx);
  hal_spi_transfer32(ty);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4 + 4);
}
ft_void hal_spi_cmd_stop()
{
  hal_spi_wrcmdstart(4);
  hal_spi_transfer32(CMD_STOP);
  hal_spi_end();
  hal_spi_updatecmdfifo(4);
}
ft_void hal_spi_cmd_slider(ft_int16_t x, ft_int16_t y, ft_int16_t w, ft_int16_t h, ft_uint16_t options, ft_uint16_t val, ft_uint16_t range)
{
  hal_spi_wrcmdstart(4 + 2 + 2 + 2 + 2 + 2 + 2 + 2);
  hal_spi_transfer32(CMD_SLIDER);
  hal_spi_transfer16(x);
  hal_spi_transfer16(y);
  hal_spi_transfer16(w);
  hal_spi_transfer16(h);
  hal_spi_transfer16(options);
  hal_spi_transfer16(val);
  hal_spi_transfer16(range);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 2 + 2 + 2 + 2 + 2 + 2 + 2);
}
ft_void cmd_touch_transform(ft_int32_t x0, ft_int32_t y0, ft_int32_t x1, ft_int32_t y1, ft_int32_t x2, ft_int32_t y2, ft_int32_t tx0, ft_int32_t ty0, ft_int32_t tx1, ft_int32_t ty1, ft_int32_t tx2, ft_int32_t ty2, ft_uint16_t result)
{
  hal_spi_wrcmdstart(4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 2);
  hal_spi_transfer32(CMD_TOUCH_TRANSFORM);
  hal_spi_transfer32(x0);
  hal_spi_transfer32(y0);
  hal_spi_transfer32(x1);
  hal_spi_transfer32(y1);
  hal_spi_transfer32(x2);
  hal_spi_transfer32(y2);
  hal_spi_transfer32(tx0);
  hal_spi_transfer32(ty0);
  hal_spi_transfer32(tx1);
  hal_spi_transfer32(ty1);
  hal_spi_transfer32(tx2);
  hal_spi_transfer32(ty2);
  hal_spi_transfer16(result);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 2 + 3);
}
ft_void hal_spi_cmd_interrupt(ft_uint32_t ms)
{
  hal_spi_wrcmdstart(4 + 4);
  hal_spi_transfer32(CMD_INTERRUPT);
  hal_spi_transfer32(ms);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4);
}
ft_void hal_spi_cmd_fgcolor(ft_uint32_t c)
{
  hal_spi_wrcmdstart(4 + 4);
  hal_spi_transfer32(CMD_FGCOLOR);
  hal_spi_transfer32(c);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4);
}
ft_void hal_spi_cmd_rotate(ft_int32_t a)
{
  hal_spi_wrcmdstart(4 + 4);
  hal_spi_transfer32(CMD_ROTATE);
  hal_spi_transfer32(a);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4);
}
ft_void hal_spi_cmd_button_sram(ft_int16_t x, ft_int16_t y, ft_int16_t w, ft_int16_t h, ft_int16_t font, ft_uint16_t options, ft_char8_t* s)
{
  hal_spi_wrcmdstart(4 + 2 + 2 + 2 + 2 + 2 + 2 + strlen(s) + 1);
  hal_spi_transfer32(CMD_BUTTON);
  hal_spi_transfer16(x);
  hal_spi_transfer16(y);
  hal_spi_transfer16(w);
  hal_spi_transfer16(h);
  hal_spi_transfer16(font);
  hal_spi_transfer16(options);
  hal_spi_transfer_string(s);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 2 + 2 + 2 + 2 + 2 + 2 + strlen(s) + 1);
}

ft_void hal_spi_cmd_button(ft_int16_t x, ft_int16_t y, ft_int16_t w, ft_int16_t h, ft_int16_t font, ft_uint16_t options, const ft_char8_t* s)
{
  hal_spi_wrcmdstart(4 + 2 + 2 + 2 + 2 + 2 + 2 + hal_strlen(s) + 1);
  hal_spi_transfer32(CMD_BUTTON);
  hal_spi_transfer16(x);
  hal_spi_transfer16(y);
  hal_spi_transfer16(w);
  hal_spi_transfer16(h);
  hal_spi_transfer16(font);
  hal_spi_transfer16(options);
  hal_spi_transfer_string_const(s);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 2 + 2 + 2 + 2 + 2 + 2 + hal_strlen(s) + 1);
}
ft_void hal_spi_cmd_memwrite(ft_uint32_t ptr, ft_uint32_t num)
{
  hal_spi_wrcmdstart(4 + 4 + 4);
  hal_spi_transfer32(CMD_MEMWRITE);
  hal_spi_transfer32(ptr);
  hal_spi_transfer32(num);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4 + 4);
}
ft_void hal_spi_cmd_scrollbar(ft_int16_t x, ft_int16_t y, ft_int16_t w, ft_int16_t h, ft_uint16_t options, ft_uint16_t val, ft_uint16_t size, ft_uint16_t range)
{
  hal_spi_wrcmdstart(4 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2);
  hal_spi_transfer32(CMD_SCROLLBAR);
  hal_spi_transfer16(x);
  hal_spi_transfer16(y);
  hal_spi_transfer16(w);
  hal_spi_transfer16(h);
  hal_spi_transfer16(options);
  hal_spi_transfer16(val);
  hal_spi_transfer16(size);
  hal_spi_transfer16(range);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2);
}
ft_void hal_spi_cmd_getmatrix(ft_int32_t a, ft_int32_t b, ft_int32_t c, ft_int32_t d, ft_int32_t e, ft_int32_t f)
{
  hal_spi_wrcmdstart(4 + 4 + 4 + 4 + 4 + 4 + 4);
  hal_spi_transfer32(CMD_GETMATRIX);
  hal_spi_transfer32(a);
  hal_spi_transfer32(b);
  hal_spi_transfer32(c);
  hal_spi_transfer32(d);
  hal_spi_transfer32(e);
  hal_spi_transfer32(f);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4 + 4 + 4 + 4 + 4 + 4);
}
ft_void hal_spi_cmd_sketch(ft_int16_t x, ft_int16_t y, ft_uint16_t w, ft_uint16_t h, ft_uint32_t ptr, ft_uint16_t format)
{
  hal_spi_wrcmdstart(4 + 2 + 2 + 2 + 2 + 4 + 2);
  hal_spi_transfer32(CMD_SKETCH);
  hal_spi_transfer16(x);
  hal_spi_transfer16(y);
  hal_spi_transfer16(w);
  hal_spi_transfer16(h);
  hal_spi_transfer32(ptr);
  hal_spi_transfer16(format);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 2 + 2 + 2 + 2 + 4 + 2);
}
ft_void hal_spi_cmd_memset(ft_uint32_t ptr, ft_uint32_t value, ft_uint32_t num)
{
  hal_spi_wrcmdstart(4 + 4 + 4 + 4);
  hal_spi_transfer32(CMD_MEMSET);
  hal_spi_transfer32(ptr);
  hal_spi_transfer32(value);
  hal_spi_transfer32(num);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4 + 4 + 4);
}
ft_void hal_spi_cmd_gradcolor(ft_uint32_t c)
{
  hal_spi_wrcmdstart(4 + 4);
  hal_spi_transfer32(CMD_GRADCOLOR);
  hal_spi_transfer32(c);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4);
}
ft_void hal_spi_cmd_bitmap_transform(ft_int32_t x0, ft_int32_t y0, ft_int32_t x1, ft_int32_t y1, ft_int32_t x2, ft_int32_t y2, ft_int32_t tx0, ft_int32_t ty0, ft_int32_t tx1, ft_int32_t ty1, ft_int32_t tx2, ft_int32_t ty2, ft_uint16_t result)
{
  hal_spi_wrcmdstart(4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 2);
  hal_spi_transfer32(CMD_BITMAP_TRANSFORM);
  hal_spi_transfer32(x0);
  hal_spi_transfer32(y0);
  hal_spi_transfer32(x1);
  hal_spi_transfer32(y1);
  hal_spi_transfer32(x2);
  hal_spi_transfer32(y2);
  hal_spi_transfer32(tx0);
  hal_spi_transfer32(ty0);
  hal_spi_transfer32(tx1);
  hal_spi_transfer32(ty1);
  hal_spi_transfer32(tx2);
  hal_spi_transfer32(ty2);
  hal_spi_transfer16(result);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 2);
}
ft_void hal_spi_cmd_calibrate(ft_uint32_t result)
{
  hal_spi_wrcmdstart(4 + 4);
  hal_spi_transfer32(CMD_CALIBRATE);
  hal_spi_transfer32(result);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4);
}
ft_void hal_spi_cmd_setfont(ft_uint32_t font, ft_uint32_t ptr)
{
  hal_spi_wrcmdstart(4 + 4 + 4);
  hal_spi_transfer32(CMD_SETFONT);
  hal_spi_transfer32(font);
  hal_spi_transfer32(ptr);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4 + 4);
}
ft_void hal_spi_cmd_logo()
{
  hal_spi_wrcmdstart(4);
  hal_spi_transfer32(CMD_LOGO);
  hal_spi_end();
  hal_spi_updatecmdfifo(4);
}
ft_void hal_spi_cmd_append(ft_uint32_t ptr, ft_uint32_t num)
{
  hal_spi_wrcmdstart(4 + 4 + 4);
  hal_spi_transfer32(CMD_APPEND);
  hal_spi_transfer32(ptr);
  hal_spi_transfer32(num);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4 + 4);
}
ft_void hal_spi_cmd_memzero(ft_uint32_t ptr, ft_uint32_t num)
{
  hal_spi_wrcmdstart(4 + 4 + 4);
  hal_spi_transfer32(CMD_MEMZERO);
  hal_spi_transfer32(ptr);
  hal_spi_transfer32(num);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4 + 4);
}
ft_void hal_spi_cmd_scale(ft_int32_t sx, ft_int32_t sy)
{
  hal_spi_wrcmdstart(4 + 4 + 4);
  hal_spi_transfer32(CMD_SCALE);
  hal_spi_transfer32(sx);
  hal_spi_transfer32(sy);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4 + 4);
}
ft_void hal_spi_cmd_clock(ft_int16_t x, ft_int16_t y, ft_int16_t r, ft_uint16_t options, ft_uint16_t h, ft_uint16_t m, ft_uint16_t s, ft_uint16_t ms)
{
  hal_spi_wrcmdstart(4 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2);
  hal_spi_transfer32(CMD_CLOCK);
  hal_spi_transfer16(x);
  hal_spi_transfer16(y);
  hal_spi_transfer16(r);
  hal_spi_transfer16(options);
  hal_spi_transfer16(h);
  hal_spi_transfer16(m);
  hal_spi_transfer16(s);
  hal_spi_transfer16(ms);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2);
}
ft_void hal_spi_cmd_gradient(ft_int16_t x0, ft_int16_t y0, ft_uint32_t rgb0, ft_int16_t x1, ft_int16_t y1, ft_uint32_t rgb1)
{
  hal_spi_wrcmdstart(4 + 2 + 2 + 4 + 2 + 2 + 4);
  hal_spi_transfer32(CMD_GRADIENT);
  hal_spi_transfer16(x0);
  hal_spi_transfer16(y0);
  hal_spi_transfer32(rgb0);
  hal_spi_transfer16(x1);
  hal_spi_transfer16(y1);
  hal_spi_transfer32(rgb1);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 2 + 2 + 4 + 2 + 2 + 4);
}
ft_void hal_spi_cmd_setmatrix()
{
  hal_spi_wrcmdstart(4);
  hal_spi_transfer32(CMD_SETMATRIX);
  hal_spi_end();
  hal_spi_updatecmdfifo(4);
}
ft_void hal_spi_cmd_track(ft_int16_t x, ft_int16_t y, ft_int16_t w, ft_int16_t h, ft_int16_t tag)
{
  hal_spi_wrcmdstart(4 + 2 + 2 + 2 + 2 + 2);
  hal_spi_transfer32(CMD_TRACK);
  hal_spi_transfer16(x);
  hal_spi_transfer16(y);
  hal_spi_transfer16(w);
  hal_spi_transfer16(h);
  hal_spi_transfer16(tag);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 2 + 2 + 2 + 2 + 2);
}
ft_void hal_spi_cmd_getptr(ft_uint32_t result)
{
  hal_spi_wrcmdstart(4 + 4);
  hal_spi_transfer32(CMD_GETPTR);
  hal_spi_transfer32(result);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4);
}
ft_void hal_spi_cmd_progress(ft_int16_t x, ft_int16_t y, ft_int16_t w, ft_int16_t h, ft_uint16_t options, ft_uint16_t val, ft_uint16_t range)
{
  hal_spi_wrcmdstart(4 + 2 + 2 + 2 + 2 + 2 + 2 + 2);
  hal_spi_transfer32(CMD_PROGRESS);
  hal_spi_transfer16(x);
  hal_spi_transfer16(y);
  hal_spi_transfer16(w);
  hal_spi_transfer16(h);
  hal_spi_transfer16(options);
  hal_spi_transfer16(val);
  hal_spi_transfer16(range);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 2 + 2 + 2 + 2 + 2 + 2 + 2);
}
ft_void hal_spi_cmd_coldstart()
{
  hal_spi_wrcmdstart(4);
  hal_spi_transfer32(CMD_COLDSTART);
  hal_spi_end();
  hal_spi_updatecmdfifo(4);
}
ft_void hal_spi_cmd_keys(ft_int16_t x, ft_int16_t y, ft_int16_t w, ft_int16_t h, ft_int16_t font, ft_uint16_t options, const ft_char8_t* s)
{
  ft_int16_t Length;
  Length = hal_strlen(s);
  hal_spi_wrcmdstart(4 + 2 + 2 + 2 + 2 + 2 + 2 + Length + 1);
  hal_spi_transfer32(CMD_KEYS);
  hal_spi_transfer16(x);
  hal_spi_transfer16(y);
  hal_spi_transfer16(w);
  hal_spi_transfer16(h);
  hal_spi_transfer16(font);
  hal_spi_transfer16(options);
  hal_spi_transfer_string_const(s);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 2 + 2 + 2 + 2 + 2 + 2 + Length + 1);
}
ft_void hal_spi_cmd_dial(ft_int16_t x, ft_int16_t y, ft_int16_t r, ft_uint16_t options, ft_uint16_t val)
{
  hal_spi_wrcmdstart(4 + 2 + 2 + 2 + 2 + 2);
  hal_spi_transfer32(CMD_DIAL);
  hal_spi_transfer16(x);
  hal_spi_transfer16(y);
  hal_spi_transfer16(r);
  hal_spi_transfer16(options);
  hal_spi_transfer16(val);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 2 + 2 + 2 + 2 + 2);
}
ft_void hal_spi_cmd_loadimage(ft_uint32_t ptr, ft_uint32_t options)
{
  hal_spi_wrcmdstart(4 + 4 + 4);
  hal_spi_transfer32(CMD_LOADIMAGE);
  hal_spi_transfer32(ptr);
  hal_spi_transfer32(options);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4 + 4);
}
ft_void hal_spi_cmd_dlstart()
{
  hal_spi_wrcmdstart(4);
  hal_spi_transfer32(CMD_DLSTART);
  hal_spi_end();
  hal_spi_updatecmdfifo(4);
}
ft_void hal_spi_cmd_snapshot(ft_uint32_t ptr)
{
  hal_spi_wrcmdstart(4 + 4);
  hal_spi_transfer32(CMD_SNAPSHOT);
  hal_spi_transfer32(ptr);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4);
}
ft_void hal_spi_cmd_screensaver()
{
  hal_spi_wrcmdstart(4);
  hal_spi_transfer32(CMD_SCREENSAVER);
  hal_spi_end();
  hal_spi_updatecmdfifo(4);
}
ft_void hal_spi_cmd_memcrc(ft_uint32_t ptr, ft_uint32_t num, ft_uint32_t result)
{
  hal_spi_wrcmdstart(4 + 4 + 4 + 4);
  hal_spi_transfer32(CMD_MEMCRC);
  hal_spi_transfer32(ptr);
  hal_spi_transfer32(num);
  hal_spi_transfer32(result);
  hal_spi_end();
  hal_spi_updatecmdfifo(4 + 4 + 4 + 4);
}


