/*
* ============================================================================
* History
* =======
*
* (C) Copyright 2014, Future Technology Devices International Ltd.
* ============================================================================
*
* This source code ("the Software") is provided by Future Technology Devices
* International Limited ("FTDI") subject to the licence terms set out
* http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
* You must read the Licence Terms before downloading or using the Software.
* By installing or using the Software you agree to the Licence Terms. If you
* do not agree to the Licence Terms then do not download or use the Software.
*
* Without prejudice to the Licence Terms, here is a summary of some of the key
* terms of the Licence Terms (and in the event of any conflict between this
* summary and the Licence Terms then the text of the Licence Terms will
* prevail).
*
* The Software is provided "as is".
* There are no warranties (or similar) in relation to the quality of the
* Software. You use it at your own risk.
* The Software should not be used in, or for, any medical device, system or
* appliance. There are exclusions of FTDI liability for certain types of loss
* such as: special loss or damage; incidental loss or damage; indirect or
* consequential loss or damage; loss of income; loss of business; loss of
* profits; loss of revenue; loss of contracts; business interruption; loss of
* the use of money or anticipated savings; loss of information; loss of
* opportunity; loss of goodwill or reputation; and/or loss of, damage to or
* corruption of data.
* There is a monetary cap on FTDI's liability.
* The Software may have subsequently been amended by another user and then
* distributed by that other user ("Adapted Software").  If so that user may
* have additional licence terms that apply to those amendments. However, FTDI
* has no liability in relation to those amendments.
* ============================================================================
*/

#include <stdio.h>

/* Data types inclusion */
#include "FT_DataTypes.h"
/* FT800 specific inclusion */
#include "FT_Gpu.h"
/* FT800 coprocessor specific inclusion */
#include "FT_CoPro_Cmds.h"
/* FT800 SPI HAL specific inclusion */
#include "FT_Hal_Spi.h"
/* Sample application include */
#include "SampleApp.h"

extern ft_uint32_t hal_gpu_dli;

/* Main entry point */
/* Boot up for FT800 followed by graphics primitive sample cases */
const ft_uint8_t FT_DLCODE_BOOTUP[12] = 
{
	0,255,0,2,
	7,0,0,38,
    	0,0,0,0,
};


ft_void	SAMAPP_GPU_Points()
{
	// Construct DL of points 
	hal_gpu_dli=0;

	hal_gpu_dl(CLEAR(1,1,1));			// Clear the screen
	hal_gpu_dl(COLOR_RGB(128, 0, 0) );		// Set the draw color to Red
	hal_gpu_dl(POINT_SIZE(5 * 16) );		// Set size to 5 * 16 /16 = 5 pixels
	hal_gpu_dl(BEGIN(FTPOINTS) );			// Start the point draw
	hal_gpu_dl(VERTEX2F(30 * 16, 17 * 16) );	// Draw circle 30 pixels from left and 17 down
	hal_gpu_dl(COLOR_RGB(0, 128, 0) );		// Set the draw color to Green
	hal_gpu_dl(POINT_SIZE(8 * 16) );		// Set size to 8 * 16 /16 = 8 pixels
	hal_gpu_dl(VERTEX2F(90 * 16, 17 * 16) );	// Draw circle 90 pixels from left and 17 down
	hal_gpu_dl(COLOR_RGB(0, 0, 128) );		// Set the draw color to Blue
	hal_gpu_dl(POINT_SIZE(10 * 16) );		// Set size to 10 * 16 /16 = 10 pixels
	hal_gpu_dl(VERTEX2F(30 * 16, 51 * 16) );	// Draw circle 30 pixels from left and 51 down	
	hal_gpu_dl(COLOR_RGB(128, 128, 0) );		// Set the draw color to Yellow
	hal_gpu_dl(POINT_SIZE(13 * 16) );		// Set size to 13 * 16 /16 = 13 pixels
	hal_gpu_dl(VERTEX2F(90 * 16, 51* 16) );		// Draw circle 90 pixels from left and 51 down
	hal_gpu_dl(DISPLAY());				// End the display list
	hal_spi_wr8(REG_DLSWAP,DLSWAP_FRAME);		// Make this display list active on the next frame
}


ft_int32_t main()
{
	ft_uint8_t DLSwapMode=DLSWAP_FRAME; 
	ft_uint8_t Pclk = 5, chipid;
	ft_uint32_t chipidrom;

	/* SPI HAL initialization */
	hal_spi_init();

	/* Do a core reset for safer side */
	hal_spi_corerst();

	hal_spi_rd16(RAM_G);

	//confirm that ftvc is up
	chipid = hal_spi_rd8(REG_ID);
	// change the clock to external clock and set the PLL to 48Mhz 
	hal_spi_clkconfig(); // 20secs

	/* Read the register id from ft800 */
	chipidrom = hal_spi_rd32(0x000C0000);

	/* enable backlight display by setting GPIO7 */
	/* Enable audio - Audio amplifier is swithed off by writting 1 to bit1 of GPIO */
	hal_spi_wr8(REG_GPIO,0xFF);//look into data sheet for the various configurations of GPIOs

	/* Boot up sequence */
	/* Clock is configured in the above code - optional if external clock is not avilable */
	/* By default the display settings are for WQVGA, so first download the first DL into DL memory */
	/* Write FRAME_SWAP into DL_SWAP register */
	/* Followed by enabling PCLK - note that PCLK for WQVGA is 5 and QVGA is 8 - follow DS/PG/AN for more information */
	hal_spi_wr8s(RAM_DL,(ft_uint8_t *)FT_DLCODE_BOOTUP,12);
	hal_spi_wr8(REG_DLSWAP,DLSWAP_FRAME);
	hal_spi_wr8(REG_PCLK,Pclk);
	/* Touch configuration - configure the resistance value to 1200 - this value is specific to customer requirement and derived by experiment */
	hal_spi_wr16(REG_TOUCH_RZTHRESH,1200);
	
	SAMAPP_GPU_Points();
	hal_spi_exit();
}




